@@ -6,6 +6,7 @@ This module contains functionality for access to PolyAnalyst Analytical Client A
 """
 import datetime
 import functools
+import math
 import time
 import warnings
 from urllib.parse import urlparse, parse_qs
@@ -96,6 +97,10 @@ class Project:
           ...     'Federated Search',
           ...     prj.get_node_list()[1],
           ... )
+
+        or, if you want to execute all nodes, call this method with no ``args``:
+
+          >>> prj.execute()
         """
         nodes = []
         for arg in args:
@@ -157,7 +162,7 @@ class Project:
 
         .. versionadded:: 0.18.0
         """
-        return Parameters(self.api, self.uuid, self._find_node(name)['id'])
+        return Parameters(self, self._find_node(name)['id'])
 
     def unload(self) -> None:
         """Unload the project from the memory and free system resources."""
@@ -195,20 +200,31 @@ class Project:
 
         raise APIException(f"Node not found: name='{name_}', type='{type_}'", status_code=500)
 
-    def wait_for_completion(self, node: Union[str, Dict[str, str]]) -> bool:
-        """Waits for the node to complete the execution. Returns True if node have
-        completed successfully and False otherwise.
+    def wait_for_completion(self, node: Union[str, Dict[str, str]], wave_id: Optional[int] = None) -> bool:
+        """
+        Waits for the node in a sequence of nodes to complete. Returns `True` if
+        node have completed successfully and `False` otherwise.
+
+        Unlike `execute(..., wait=True)`, which returns only after an entire node
+        sequence has completed, this method returns immediately after the specified
+        node has completed.
+
+        :param node: Node name or dict with name and type of node that runs within execution wave
+        :param wave_id: Execution wave identifier
 
         .. deprecated:: 0.17.0
            Use :meth:`Project.is_running` instead.
 
-        :param node: node name or dict with name and type of node
+        .. versionchanged:: 0.23.0
+           Introduced this deprecated method back. Added `wave_id` argument.
         """
-        warnings.warn(
-            'Project.wait_for_completion() is deprecated, use Project.is_running() instead.',
-            DeprecationWarning,
-            stacklevel=2,
-        )
+        if not wave_id:
+            warnings.warn(
+                'It is recommended to use `Project.wait_for_completion` with `wave_id` argument',
+                UserWarning,
+                stacklevel=2,
+            )
+
         time.sleep(0.5)  # give pa time to update node statuses
         while True:
             self._update_node_list()
@@ -216,9 +232,11 @@ class Project:
 
             if stats.get('errMsg'):
                 return False
-            if stats['status'] == 'synchronized':
+            elif stats['status'] == 'synchronized':
                 return True
-            if stats['status'] == 'incomplete':
+            elif stats['status'] == 'incomplete':
+                return False
+            elif wave_id and not self.is_running(wave_id):
                 return False
 
             time.sleep(1)
@@ -251,7 +269,8 @@ class Project:
            Use :meth:`Project.get_execution_stats` instead.
         """
         warnings.warn(
-            'Project.get_execution_statistics() is deprecated, use Project.get_execution_stats() instead.',
+            'Project.get_execution_statistics() is deprecated, use'
+            'Project.get_execution_stats() instead.',
             DeprecationWarning,
             stacklevel=2,
         )
@@ -276,19 +295,16 @@ class Project:
         return self.dataset(node).preview()
 
     def set_parameters(
-            self,
-            node: str,
-            node_type: str,
-            parameters: Dict[str, Any],
-            declare_unsync: bool = True,
-            hard_update: bool = True,
+        self,
+        node: str,
+        node_type: str,
+        parameters: Dict[str, Any],
+        declare_unsync: bool = True,
+        hard_update: bool = True,
     ) -> None:
         """
         Set parameters of the selected Parameters node in the project.
 
-        .. deprecated:: 0.18.0
-           Use :meth:`Parameters.set` instead.
-
         :param node: name of Parameters node
         :param node_type: node type, which parameters need to be set. The types \
             are listed in NodeTypes.
@@ -296,36 +312,40 @@ class Project:
         :param declare_unsync: reset the status of the Parameters node.
         :param hard_update: update every child node with new parameters if True, \
             otherwise reset their statuses. Works only if declare_unsync is True.
+
+        .. deprecated:: 0.18.0
+           Use :meth:`Parameters.set` instead.
         """
         warnings.warn(
             'Project.set_parameters() is deprecated, use Project.parameters.set() instead.',
             DeprecationWarning,
             stacklevel=2,
         )
-        warns = self.parameters(node).set(node_type, parameters, declare_unsync, hard_update)
+        warns = self.parameters(node).set(node_type, parameters, None, declare_unsync, hard_update)
         if warns:
             for msg in warns:
                 warnings.warn(msg)
 
 
 class Parameters:
-    def __init__(self, api, uuid: Optional[str], id: Optional[str]):
-        self.api = api
-        self.uuid = uuid
-        self.id = id
+    def __init__(self, prj: Project, _id: Optional[str]):
+        self._prj = prj
+        self._api = prj.api
+        self.id = _id
 
     def get(self):
         """Returns list of nodes with parameters and strategies supported by ``Parameters`` node."""
-        return self.api.get('parameters/nodes')
+        return self._api.get('parameters/nodes')
 
     def set(
-            self,
-            node_type: str,
-            parameters: Union[Dict[str, str], List[Dict[str, str]]],
-            strategies: Optional[List[int]] = None,
-            declare_unsync: bool = True,
-            hard_update: bool = True,
-    ) -> None:
+        self,
+        node_type: str,
+        parameters: Union[Dict[str, str], List[Dict[str, str]]],
+        strategies: Optional[List[int]] = None,
+        declare_unsync: bool = True,
+        hard_update: bool = True,
+        wait: bool = True,
+    ) -> Optional[List[str]]:
         """
         Sets `node_type` parameters and strategies for the Parameters node.
 
@@ -339,6 +359,8 @@ class Parameters:
         :param hard_update: update every child node with new parameters if True, \
             otherwise reset their statuses. Works only if declare_unsync is True.\
             True by default.
+        :param wait: wait for this node to set parameters for each child node.\
+            True by default.
         """
 
         if strategies is None:
@@ -346,9 +368,10 @@ class Parameters:
 
         method = 'configure-array' if isinstance(parameters, list) else 'configure'
 
-        self.api.post(
+        resp, _warnings = self._api.request(
             f'parameters/{method}',
-            params={'prjUUID': self.uuid, 'obj': self.id},
+            method='POST',
+            params={'prjUUID': self._prj.uuid, 'obj': self.id},
             json={
                 'type': node_type,
                 'settings': parameters,
@@ -358,6 +381,18 @@ class Parameters:
             },
         )
 
+        query = urlparse(resp.headers.get('location')).query
+        try:
+            wave_id = int(parse_qs(query).get('executionWave')[0])
+        except TypeError:
+            pass
+        else:
+            if wait:
+                while self._prj.is_running(wave_id):
+                    time.sleep(1)
+
+        return _warnings
+
     def clear(self, *node_types: List[str], declare_unsync: bool = True) -> Optional[List[str]]:
         """
         Clears parameters and strategies of `node_types` for the Parameters node.
@@ -366,13 +401,13 @@ class Parameters:
         :param node_types: node types which parameters needs to be cleared
         :param declare_unsync: reset status of the Parameters node
         """
-        return self.api.post(
+        return self._api.post(
             'parameters/clear',
-            params={'prjUUID': self.uuid, 'obj': self.id},
+            params={'prjUUID': self._prj.uuid, 'obj': self.id},
             json={
                 'nodes': node_types,
                 'declareUnsync': declare_unsync,
-            }
+            },
         )
 
 
@@ -384,6 +419,7 @@ def retry_on_invalid_guid(func):
         except _WrapperNotFound:
             cls._update_guid()
             return func(cls, *args, **kwargs)
+
     return wrapper
 
 
@@ -411,10 +447,16 @@ class DataSet:
         """Returns first 1000 rows with strings truncated to 250 characters."""
         return self._api.get(
             'dataset/preview',
-            params={'prjUUID': self._prj.uuid, 'name': self._node['name'], 'type': self._node['type']},
+            params={
+                'prjUUID': self._prj.uuid,
+                'name': self._node['name'],
+                'type': self._node['type'],
+            },
         )
 
-    def iter_rows(self, start: int = 0, stop: Optional[int] = None) -> Iterator[Dict[str, JSON_VAL]]:
+    def iter_rows(
+        self, start: int = 0, stop: Optional[int] = None
+    ) -> Iterator[Dict[str, JSON_VAL]]:
         """
         Iterate over rows in dataset.
 
@@ -438,9 +480,11 @@ class DataSet:
         if stop is None:
             stop = max_row
 
-        # предпологается что если stop определен то пользователь в курсе количества строк в датасете
+        # предполагается что если stop определен то пользователь в курсе количества строк в датасете
         if not 0 <= start <= stop <= max_row:
-            raise ValueError(f'start and stop arguments must be within dataset row range: (0, {max_row})')
+            raise ValueError(
+                f'start and stop arguments must be within dataset row range: (0, {max_row})'
+            )
 
         rows = self._values(stop)['table']
         get_text = self._cell_text
@@ -459,10 +503,18 @@ class DataSet:
                 result = {}
                 for column in info['columnsInfo']:
                     if column['flags'].get('getTextAlways'):
-                        result[column['title']] = get_text(self.idx, column['id'], column['title'])
+                        result[column['title']] = get_text(self.idx, column['id'])
                     # elif column['type'] == 'DateTime':  # todo convert to python datetime?
                     else:
-                        result[column['title']] = rows[self.idx][column['id']]
+                        _value = rows[self.idx][column['id']]
+                        if _value == 1e+100:
+                            _value = None
+                        elif _value == 8e+100:
+                            _value = math.inf
+                        elif _value == -8e+100:
+                            _value = -math.inf
+
+                        result[column['title']] = _value
 
                 self.idx += 1
                 return result
@@ -477,19 +529,18 @@ class DataSet:
 
     @retry_on_invalid_guid
     def _values(self, row_count: int) -> Dict[str, Union[List, Dict]]:
-        return self._api.get('dataset/values', json={'wrapperGuid': self.guid, 'rowCount': row_count})
+        return self._api.get(
+            'dataset/values',
+            json={'wrapperGuid': self.guid, 'rowCount': row_count},
+        )
 
     @retry_on_invalid_guid
-    def _cell_text(self, row: int, col: int, _title) -> str:
+    def _cell_text(self, row: int, col: int) -> str:
         return self._api.get(
             'dataset/cell-text',
             json={
                 'wrapperGuid': self.guid,
                 'row': row,
                 'col': col,
-                # todo remove next keys
-                'colTitle': _title,
-                'offset': 0,
-                'count': 0,
             },
         )['text']
